<%--
 * Component: iface
 *
 * Description:
 *   interface to initiate retrival from archive
 *
 * Parameters:
 *
 *        [LEGACY FOR TEST SCRIPTS - cOper=retrieve is implied]
 *          cStudyUid   : identifies the study to be retrieved
 *          cDevice     : device id of the archive device where the study is archived
 *
 *        [Common]
 *          cOper       : archive operation (currently the only supported one is 'retrieve'
 *
 *        [For a single study preceded by an archivelist.jsp]
 *          cStudy      : identifies the study to be retrieved
 *          cDevice     : device id of the archive device where the study is archived
 *
 *
 *        [multiple studies or request not preceded by an archivelist.jsp]
 *        Generic study identifier parameters like
 *          SYUI        : comma separated list of study UIDs
 *          ACCN        : comma separated list of accession numbers
 *          [PAID]      : optional patient ID used when searching studies by accession number
 *
--%><%@
	page import="java.util.*, imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType,
			imagemed.imsession.*, imagemed.exception.*, imagemed.archive.ArchiveRetrieve"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%
	IMSession is = new IMSession( request, response, out );
	is.response.setContentType( "text/plain" );

	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	}

	if ( !is.hasRight( imagemed.types.flag.IMTypeur.ARCHIVE_RIGHT ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to perform this operation." );
		return;
	}

	String study = request.getParameter( "cStudy" );
	if ( "".equals( study ) ) {
		study = null;
	}
	String oper = request.getParameter( "cOper" );
	if ( "".equals( oper ) ) {
		oper = null;
	}
	String device = request.getParameter( "cDevice" );
	if ( "".equals( device ) ) {
		device = null;
	}
	
	if ( study == null && oper == null ) {
		// TODO: to be removed on long term
		// Check for legacy parameters used by test scripts
		study = request.getParameter( "cStudyUID" );
		if ( "".equals( study ) ) {
			study = null;
		}
		if ( study != null ) {
			oper = "retrieve";
		}
	}

	if ( oper == null ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cOper'." );
		return;
	}
	if ( !"retrieve".equals( oper ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Unrecognized operation: '" + oper + "'" );
		return;
	}

	Properties studies = new Properties();
	
	if ( study == null ) {
		try {
			StudyFinder sf = new StudyFinder( is, false, true );
			HashMap<String, StudyFinder.StudyLocation> locations = sf.collectStudyLocations( StudyFinder.StudyIdentifier.UID, false );
			for ( String sty : locations.keySet() ) {
				StudyFinder.StudyLocation info = locations.get( sty );
				for ( String loc : info.getLocations() ) {
					if ( "".equals( loc ) ) {
						// Should not happen as worklist is not queried, but for safety
						continue;
					}
					if ( device != null && ! device.equals( loc ) ) {
						// Only retreive from requested archive - but this is not the one
						continue;
					}
					studies.setProperty( sty, loc );
				}
			}
		}
		catch ( IMIfaceException ex ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "Missing study identifier parameters." );
			return;
		}

		if ( studies.isEmpty() ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "No such study in the archive." );
			return;
		}
	}
	else if ( device == null ) {
		// cStudy was provided but cDevice is missing
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cDevice'." );
		return;
	}
	else {
		studies.setProperty( study, device );
	}
	
	
	ArchiveRetrieve archiveRetrieve = new ArchiveRetrieve( is, null, null, false );
	String error = archiveRetrieve.getStudies( studies );
	if ( error != null && error.length() > 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, error );
		return;
	}
	
	IfaceToolbox.printOK( is );
	
	if ( true ) {
		return;
	}
%>