<%--
 * Component: iface
 *
 * Description:
 *   Queries information about frames of an object
 *
 * Parameters:
 *
 * cObj        SOP IUID of the object to query.
 * cStudy			 Required for object-less CWL only! - the StudyIUID of the study
 *
 * When successful, it returns the following values in a tab separated line:
 *       frame number, columns, rows, color type, bytes/pixel, rescale intercept, rescale slope, window, level
 *                                    mono         1, 2
 *                                    color         3
 *
--%><%@
	page import="java.sql.*,
			imagemed.dcprocessing.*, imagemed.dotcom.DotcomConfig, imagemed.imsession.*,
			imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType"

%><%
	IMSession is = new IMSession( request, response, out );

	// Check login
	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	}

	String parObj = request.getParameter( "cObj" );
	String parStudy = request.getParameter( "cStudy" );

	if ( parObj == null ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cObj'." );
		return;
	}

	String study = null;
	String objectName = null;

	if ( DotcomConfig.selfDevice().isNoObjectTable() ) {
		study = parStudy;
	}
	else {
		try {
			ResultSet rs = IMEnv.getDB().prepareStatement( "select STYIUID, FNAME from " + IMEnvBase.getEnv( "OBJECTTBL" ) +
					" where SOPIUID='" + parObj + "'" ).executeQuery();
			if ( rs.next() ) {
				study = rs.getString( "STYIUID" );
				objectName = rs.getString( "FNAME" );
			}
			rs.close();
		}
		catch ( SQLException sqlEx ) {
		}
	}

	if ( study == null ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Study was not found." );
		return;
	}

	// in fact it would be enough to redirect until the first non-objectless CWL (or HUB)
	String redirHost = RedirectToolbox.getRedirectForStudy( is, study );

	if ( redirHost != null ) {
		String redirURL = redirHost + "/iface/frameInfo.jsp?" + RedirectToolbox.getRedirectParams( is );
		response.sendRedirect( redirURL );
		return;
	}

	if ( objectName == null ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Object was not found." );
		return;
	}

	DescFile descFile = new DescFile( study, objectName );

	response.setContentType( "text/plain" );
	out.println( "#USERID: " + is.regID );
	out.println( "#RESULT: OK" );

	try {
		descFile.load();
		for ( int i = 0; i < descFile.size(); i++ ) {
			DescEntry entry = descFile.elementAt( i );
			boolean isColor = "bypixel".equals( entry.getImageType() );
			String color = isColor ? "color" : "mono";
			int bytes = entry.getDepth() / 8;
			if ( isColor )
				bytes *= 3;

			out.println( "" +
					entry.getFrameNumber() + "\t" +
					entry.getColumns() + "\t" +
					entry.getRows() + "\t" +
					color + "\t" +
					bytes + "\t" +
					entry.getActual() + "\t" +
					entry.getRescaleIntercept() + "\t" +
					entry.getRescaleSlope() + "\t" +
					entry.getWindowWidth() + "\t" +
					entry.getWindowCenter() );
		}
	}
	catch ( Throwable th ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Error retrieving frame information (" + th.toString() + ")." );
		return;
	}


	if ( true ) {
		return;
	}
%>