<%--
 * Component: iface
 *
 * Description:
 *   interface to query/update the content of UserLists
 *
 * Parameters:
 *
 *		common Parameters:
 *				cOper		: operation ("set" or "get" or "delete")
 *				cUser		: userID of the list's owner (defaults to the user calling the page)
 *				cList		: name of the list
 *				cAction  : action name
 *
 *		params for "get" (NOT YET IMPLEMENTED):
 *				----
 *		returns			: settings for the specified action
 *
 *		params for "set": 
 *            cEnabled        : indicates whether the action is enabled
 *            cApplyToCurrent : apply action to current content of the list
 *            cSchedule       : when to run the action
 *                              (possible values: 'immadiately' or 'daily')
 *            cSchedTime      : hour of the action to be started (if cSchedule is daily)
 *            action specific parameters; see createParamInput() methods of classes inherited from imagemed.actions.DcAction.
 *		returns			: none
 *
 *		params for "delete":
 *				----
 *		returns			: none
 *
--%><%@
	page import="java.io.*,
			imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType,
			imagemed.imsession.*, imagemed.actions.*, imagemed.lists.*"

%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%!
	/**
	 *	this method returns true if the user has admin rights or is identical to the given one
	 *	(for example he want's to alter his own folder or has admin right to alter someone else's...)
	 */
	private boolean hasRight( IMSession is, String user ) throws IOException {
		if ( IMAdminToolbox.bAdminRight( is ) ) {
			return true;
		}
		if ( user != null && user.equals( is.regID ) ) {
			return true;
		}
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to perform this operation." );
		return false;
	}

%><%
	IMSession is = new IMSession( request, response, out );

	is.response.setContentType( "text/plain" );

	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	}

	String oper = request.getParameter( "cOper" );
	if ( oper == null || oper.length() == 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cOper'." );
		return;
	}

	String list = request.getParameter( "cList" );
	if ( list == null || list.length() == 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cList'." );
		return;
	}

	String user = request.getParameter( "cUser" );
	if ( user == null || user.length() == 0 ) {
		user = is.regID;
	}
	else if ( !hasRight( is, user ) ) {
		return;
	}

	String action = request.getParameter( "cAction" );
	DcAction dcAction = null;
	if ( !"list".equals( oper ) ) {
		if ( action == null || action.length() == 0 ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cAction'." );
			return;
		}

		dcAction = DcActionManager.getActionForName( action );
		dcAction.setList( user, list );
	}

	//	if (oper.equals("get")) {
%><%--//		TODO: it would require dcAction.toString() to be implemented --%><%
	//		IfaceToolbox.printResult(is, dcAction.toString());
	//		return;
	//	}

	if ( oper.equals( "set" ) ) {
		dcAction.processInput( is, is.imRequest.getParTable(), is.regID );
		IfaceToolbox.printOK( is );
		return;
	}

	if ( oper.equals( "delete" ) ) {
		GenericUserList.proliferateFile( dcAction.delete() );
		IfaceToolbox.printOK( is );
		return;
	}

	IfaceToolbox.printError( is, ErrorType.Permanent, "Unrecognized operation: '" + oper + "'." );

%>