<%--
 * Component: iface
 *
 * Description:
 *   interface to query/update the content of UserLists
 *
 * Parameters:
 *
 *		common Parameters:
 *				cOper		: operation ("get" or "store" or "delete")
 *				cUser		: userID of the list's owner (defaults to the user calling the page)
 *				cList		: name of the list
 *
 *		params for "get":
 *				---
 *		returns			: content of list-file
 *
 *		params for "store": 
 *				cContent				: the content of the list-file (should be presented URLEncoded)
 *					OR
 *				cFile						: the uploaded file that with the desired content of the list file
 *		returns			: none
 *
 *		params for "delete":
 *				---
 *		returns			: none
 *
--%><%@
	page import="java.io.*,
			imagemed.actions.*, imagemed.imsession.*, imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType,
			imagemed.util.*,imagemed.lists.*"

%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%!

	/*
	 *	this method returns true if the user has admin rights or is identical to the given one
	 *	(for example he want's to alter his own folder or has admin right to alter someone else's...)
	 */
	private boolean hasRight( IMSession is, String user ) throws IOException {
		if ( IMAdminToolbox.bAdminRight( is ) ) {
			return true;
		}
		if ( user != null && user.equals( is.regID ) ) {
			return true;
		}
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to perform this operation." );
		return false;
	}

%><%
	IMSession is = new IMSession( request, response, out );

	is.response.setContentType( "text/plain" );

	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	}
	String oper = request.getParameter( "cOper" );
	if ( oper == null || oper.length() == 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cOper'." );
		return;
	}
	String list = request.getParameter( "cList" );
	if ( list == null || list.length() == 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cList'." );
		return;
	}
	String user = request.getParameter( "cUser" );
	if ( user == null || user.length() == 0 ) {
		user = is.regID;
	}
	else if ( !hasRight( is, user ) ) {
		return;
	}
	if ( oper.equals( "get" ) ) {
		String content = Helper.readTextFile( GenericUserList.getListFileName( 0, user, list ) );
		if ( content == null ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "The list was not found." );
			return;
		}
		IfaceToolbox.printResult( is, content );
		return;
	}
	if ( oper.equals( "store" ) ) {
		String content = request.getParameter( "cContent" );
		if ( content == null ) {
			MultipartParser mpParams = new MultipartParser( is, IMEnvBase.getEnv( "TMP" ) );
			String parTmpFileName = mpParams.getParameter( "cFile" );
			if ( parTmpFileName == null || !new File( parTmpFileName ).exists() ) {
				IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cContent' or 'cFile'." );
				return;
			}
			content = Helper.readTextFile( parTmpFileName );
		}
		else {
			content = Helper.URLDecode( content );
		}
		if ( list.startsWith( "." ) && is.regRights.indexOf( "A" ) == -1 ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to save hidden lists." );
			return;
		}
		if ( list.equals( ".archive" ) && user.equals( "@system" ) ) {
			if ( is.regRights.indexOf( "@" ) == -1 ) {
				IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to save archive lists." );
				return;
			}
			UserList.backupArchiveList( is );
		}
		Helper.writeTextFile( GenericUserList.getListFileName( 0, user, list ), content );
		IfaceToolbox.printOK( is );
		return;
	}
	if ( oper.equals( "delete" ) ) {
		UserList userList = new UserList();
		//		userList.setListUserRights( is.regRights );

		//		userList.setListType(UserList.LIST_USERFILTER);
		userList.setUser( user );
		userList.setList( list );
		GenericUserList.proliferateFiles( userList.delete( is ) );

		//		File listFile = new File(UserList.getListFileName(user, list));
		//		if (listFile.exists()) {
		//			listFile.delete();
		//			IfaceToolbox.printOK(is);
		//			return;
		//		}
		//		IfaceToolbox.printError(is, "List not found!");
		return;
	}
	IfaceToolbox.printError( is, ErrorType.Permanent, "Unrecognized operation: '" + oper + "'" );

	if ( true ) {
		return;
	}
%>