<%--
 * Component: iface
 *
 * Description:
 *  Interface for Matching criteria
 *
 *  @param parOp         Operation to be performed
 *
 *  Possible values for parOp and specific params
 *
 *    merge :         modify merge.cfg
 *      @param r_<colid>   'R' - means correct study data with RIS data
 *      @param c_<colid>   correction mode of the field ('S' - soft, 'H' -hard)
 *
 *    set_matching  : modify autocorrection criteria
 *    set_matching2 : modify manual correction criteria
 *      @param m           new correction criteria
 *
 *    add_matching  : add new criterion to autocorrection criteria
 *    add_matching2 : add new criterion to manual correction criteria
 *      @param add         criterion to be added
 *
 *    prior :         modify prior matching criteria
 *      @param pr_list     prior matching fields
 *
--%><%@
	page import="java.util.*,
			imagemed.dcprocessing.*, imagemed.exception.*, imagemed.imsession.*, imagemed.jsputil.*,
			imagemed.util.*"

%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%
	IMSession is = new IMSession( request, response, out );

	try {

		if ( !IfaceToolbox.isRegOK( is ) ) {
			throw new IMIfaceException( is, "Has no registration info for this IP." );
		}

		if ( !IMAdminToolbox.bAdminRight( is ) ) {
			throw new IMIfaceException( is, "Has no Admin right for this operation." );
		}

		String parOp; // op

		parOp = is.request.getParameter( "op" );

		// Apply changes to the merge hard/soft config

		if ( "merge".equals( parOp ) ) {
			/*
			Read the content of merge config and replace editable field values
			with new values. This will keep uneditable (site specific) field
			values.
			*/

			Hashtable<String, MergeFlag> mergeConfig = Merge.readMergeConfig( IMEnvBase.getEnv( "CONF" ) + "/merge.cfg" );

			for ( Enumeration<?> e = request.getParameterNames(); e.hasMoreElements(); ) {
				String pname = (String) e.nextElement();
				if ( pname.startsWith( "r_" ) ) {
					String colid = pname.substring( 2 );
					String source = request.getParameter( pname );
					String mode = request.getParameter( "c_" + colid );

					MergeFlag mf = new MergeFlag(); // Source: Modality			
					if ( "R".equals( source ) ) {
						mf.setSource( MergeFlag.Source.RIS );
						mf.setTemporary( "D".equals( mode ) );
					}

					if ( !mf.equals( mergeConfig.get( colid ) ) ) {
						mergeConfig.put( colid, mf );
					}
				}
			} // for parameterNames


			Merge.writeMergeConfig( mergeConfig, IMEnvBase.getEnv( "CONF" ) + "/merge.cfg" );

			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: OK" );
			return;
		}

		// Apply changes to the matching criteria config
		else if ( "set_matching".equals( parOp ) || "set_matching2".equals( parOp ) ) {
			String v = request.getParameter( "m" );
			if ( "".equals( v ) )
				v = null;
			//		out.println("<PRE>\n"+v+"</pre>");
			Helper.writeTextFile( IMEnvBase.getEnv( "CONF" ) + "/" + parOp.substring( 4 ) + ".cfg", v );
			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: OK" );
			return;
		}
		else if ( "add_matching".equals( parOp ) || "add_matching2".equals( parOp ) ) {
			String v = Helper.readTextFile( IMEnvBase.getEnv( "CONF" ) + "/" + parOp.substring( 4 ) + ".cfg" );
			if ( v == null )
				v = "";

			String add[] = request.getParameterValues( "add" );
			if ( add != null ) {
				int i;
				for ( i = 0; i < add.length; i++ )
					v = v + ( i == 0 ? "" : "," ) + add[ i ];
				v = v + "\n";
			}
			//		out.println("<PRE>\n"+v+"</pre>");
			Helper.writeTextFile( IMEnvBase.getEnv( "CONF" ) + "/" + parOp.substring( 4 ) + ".cfg", v );
			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: OK" );
			return;
		}
		else if ( "prior".equals( parOp ) ) {
			String v = "";
			String pr[] = request.getParameterValues( "pr_list" );
			if ( pr != null ) {
				int i;
				for ( i = 0; i < pr.length; i++ )
					v = v + ( i == 0 ? "" : "," ) + pr[ i ];
				v = v + "\n";
			}
			v = "# configured by imadmin/merge.jsp\n" + v;
			Helper.writeTextFile( IMEnvBase.getEnv( "PRIOR_VAR" ) + "/prior.cfg", v );
			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: OK" );
			return;
		}

	}
	catch ( IMIfaceException imi ) {
		IfaceToolbox.printError( is, imi.getErrorType(), imi.getMessage() );
		return;
	}
%>