<%--
 * Component: Iface
 *
 * Description:
 *    Moves study from one HUB server to another one
 *
 *
 *  @param cStudy                  Study Instance UID of the study to be moved
 *  @param cTarget                 Device ID of the HUB server the study to be moved to
 * [@param cSession   Session id for caching study information collected from the HUB servers]
 *
 *
--%><%@
	page import="java.io.*, java.util.*,
			imagemed.dotcom.*, imagemed.dotcom.multihub.*, imagemed.exception.*, imagemed.imsession.*,
			imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType, imagemed.types.flag.IMTypeur,
			imagemed.util.*"

%><%
	//to avoid empty lines before printing out 
	out.clear();

	IMSession is = new IMSession( request, response, out );

	is.response.setHeader( "pragma", "no-cache" );
	is.response.setHeader( "Cache-Control", "no-cache, must-revalidate" );
	is.response.setHeader( "Expiry-Date", "Friday, 31-Dec-70 23:59:59 GMT" );
	is.response.setContentType( "text/plain" );

	String parStudy = request.getParameter( "cStudy" );
	String parTarget = request.getParameter( "cTarget" );
	String parSession = request.getParameter( "cSession" );

	// Gets hubs of the requesting device 
	try {

		List<String> hubs = DotcomConfig.getConfig().getHubs( DotcomConfig.selfId() );

		if ( !IfaceToolbox.isRegOK( is ) ) {
			throw new IMIfaceException( is, "Has no registration info for this IP." );
		}

		// Checking user rights
		if ( is.regRights == null || ( is.regRights.indexOf( "@" ) == -1 && is.regRights.indexOf( "A" ) == -1 ) ) {
			throw new IMIfaceException( is, "You don't have rights to view the requested page" );
		}

		if ( parStudy == null || "".equals( parStudy ) ) {
			throw new IMIfaceException( is, "Missing parameter: cStudy" );
		}

		if ( parTarget == null || "".equals( parTarget ) ) {
			throw new IMIfaceException( is, "Missing parameter: cTarget" );
		}

		boolean isNewSession = false;
		if ( parSession == null ) {
			parSession = "" + System.currentTimeMillis() + "." + Helper.getCounter();
			isNewSession = true;
		}

		StudyComparator sc = new StudyComparator( parSession, parStudy );

		try {
			if ( !sc.collectStudyData( is, !isNewSession ) ) {
				throw new IMIfaceException( is, "Study might be changed on the HUB server." );
			}
		}
		catch ( Exception ex ) {
			throw new IMIfaceException( is, "Error collecting study data from HUB servers." );
		}

		String devices[] = sc.getDevices( is );
		if ( devices == null || devices.length < 1 ) {
			throw new IMIfaceException( is, "Study data could not be collected from the owner HUB server (" + devices[ 0 ] + ")." );
		}

		if ( devices.length > 1 ) {
			// Redirect to the multiple HUB cleanup tool
			throw new IMIfaceException( is, "Needs to run multiple HUB cleanup tool" );
		}

		// Checking if the target of the is not the device that already owns the study
		if ( parTarget.equals( devices[ 0 ] ) ) {
			throw new IMIfaceException( is, "Target is the same device that the study already resides" );
		}

		// Checking if the target is a hub of the requesting device
		if ( !hubs.contains( parTarget ) ) {
			throw new IMIfaceException( is, "Target is not a hub of " + DotcomConfig.selfId() );
		}

		// Processing other parameters
		if ( parTarget != null && StudyComparator.rmiIsStudyObsoleteAt( null, parStudy, parTarget ).booleanValue() ) {
			throw new IMIfaceException( is, "The study cannot be moved to " + parTarget + ". The study is obsolete at that HUB." );
		}

		if ( !sc.lock( "Unlock", is._sid, false, is, new Vector<Properties>() ) ) {
			throw new IMIfaceException( is, "Error at Unlocking study" );
		}

		// System Lock studies
		MessageBox mbBreak = CorrectionTool.systemLock( sc, is );
		if ( !( mbBreak == null ) ) {
			if ( !sc.lock( "Unlock", is._sid, true, is, new Vector<Properties>() ) ) {
				throw new IMIfaceException( is, "Error at System Unlock Study" );
			}
		}

		int i;

		// Get object data to be prepared from HUBs the study will be removed from

		// multipart request to be sent to the HUB the study will remain on
		MultipartRequest mpr = new MultipartRequest();

		mpr.addParam( "cStudy", parStudy, MultipartRequest.MP_STRING_PARAM );
		mpr.addParam( "cSession", parSession, MultipartRequest.MP_STRING_PARAM );
		if ( is.regID != null ) {
			mpr.addParam( "cUser", is.regID, MultipartRequest.MP_STRING_PARAM );
		}

		int numTarFiles = 0;
		for ( i = 0; i < devices.length; i++ ) {
			if ( parTarget.equals( devices[ i ] ) ) {
				continue;
			}

			File dataFile = new File( IMEnvBase.getEnv( "TMP" ) + "/multihub." + parSession + "." + devices[ i ] + ".tar.gz" );
			MessageBox mb = CorrectionTool.getObjectsFrom( dataFile, parStudy, parSession, devices[ i ], true, true, null, is.regID, parTarget );

			if ( !( mb == null ) ) {
				throw new IMIfaceException( is, "Error at Multipart Request to be sent to the HUB the study will remain on" );
			}

			if ( dataFile.length() != 0 ) {
				mpr.addParam( "cHub" + numTarFiles, devices[ i ], MultipartRequest.MP_STRING_PARAM );
				mpr.addParam( "cFile" + numTarFiles, dataFile.getAbsolutePath(), MultipartRequest.MP_FILE_PARAM );
				numTarFiles++;
			}
		}

		mpr.addParam( "cNumFiles", "" + numTarFiles, MultipartRequest.MP_STRING_PARAM );
		mpr.addParam( "cDelReport", "false", MultipartRequest.MP_STRING_PARAM );
		mpr.addParam( "cMove", "true", MultipartRequest.MP_STRING_PARAM );

		// Set target device to be the owner

		MessageBox mbSetOwner = CorrectionTool.setOwnerTo( is, parTarget, parStudy );
		if ( !( mbSetOwner == null ) ) {
			throw new IMIfaceException( is, "Error at Setting target device to be the owner" );
		}

		// Put object data to the HUB the study will reside on
		MessageBox mbPutObjectsTo = CorrectionTool.putObjectsTo( is.out, parTarget, parSession, mpr );
		if ( !( mbPutObjectsTo == null ) ) {
			throw new IMIfaceException( is, "Error at Putting object data to the HUB" );
		}

		// Remove study from other HUBs
		MessageBox mbObsoleteStudy = CorrectionTool.obsoleteStudy( is, parStudy, parSession, null, is.regID, parTarget );
		if ( !( mbObsoleteStudy == null ) ) {
			throw new IMIfaceException( is, "Error setting study obsolete in other HUBs" );
		}

		// Release System lock
		try {
			sc.lock( "Unlock", is._sid, true, is, new Vector<Properties>() );
		}
		catch ( Exception ex ) {
			throw new IMIfaceException( is, "Error at unlocking Study" );
		}

	}
	catch ( IMIfaceException imi ) {
		Helper.DEBUG( "IFACE moveStudy IMIfaceException: " + Helper.throwable2String( imi ) );
		IfaceToolbox.printError( is, imi.getErrorType(), imi.getMessage() );
		return;
	}
	catch ( Throwable t ) {
		Helper.DEBUG( "IFACE moveStudy Throwable: " + Helper.throwable2String( t ) );
		IfaceToolbox.printError( is, ErrorType.Permanent, t.getMessage() );
		return;
	}

	IfaceToolbox.printOK( is, null );
%>