<%--
 * Component: iface
 *
 * Description:
 *    Returns information on studies identified by Study Instance UIDs and/or Accession Numbers
 * 
 *
 * @params:
 *
 *   ACCN   accession number of studies to query
 *          More than one accession number can be specified separated by semicolon (;)
 *   SYUI   study instance UID of studies to query
 *          More than one study can be specified separated by semicolon (;)
 *  [PAID]  patient ID, if provided all studies should belonged to this patient
 *
 *   cUser    user id
 *   cPasswd  password
 *
--%><%@
	page import="java.util.*, java.io.*,
			imagemed.exception.*, imagemed.imsession.*, imagemed.imtables.*, imagemed.jsputil.*,
			imagemed.jsputil.IfaceToolbox.ErrorType, imagemed.tables.*, imagemed.util.Helper"
	
%><%!

	static class MainStudies {
		HashMap<String, Properties> m_mainStudies;
		
		MainStudies() {
			m_mainStudies = new HashMap<String, Properties>();
		}

		boolean isMatching( Properties filters, Properties values ) {
			for ( Enumeration<?> e = filters.propertyNames(); e.hasMoreElements(); ) { 
				String field = (String) e.nextElement();
				String filter = filters.getProperty( field );
				String value = values.getProperty( field );

				if ( filter == null ) {
					// Exception ??
				}

				if ( !IMBaseFieldManager.DEFAULTFIELDMANAGER.isMatching( field, filter, value, false ) ) {
					return false;
				}
			}

			return true;
		}

		String getMainStudy( String uid, Properties filterPrp, Properties valuePrp ) {
			for ( Map.Entry<String, Properties> entry : m_mainStudies.entrySet() ) {
				String entryUid = entry.getKey();
				Properties entryFilterPrp = entry.getValue();
				
				if ( isMatching( entryFilterPrp, valuePrp ) ) {
					// The argument study is a prior of the study in the main study list
					return entryUid;
				}
			}
			
			m_mainStudies.put( uid, filterPrp );
			return uid;
		}
	}

	static void printEmpty( PrintWriter out, String uid, String accno ) throws IOException {
		out.println( "  <study>" );
		if ( uid != null ) {
			IfaceToolbox.printXMLTagln( out, "SYUI", uid );
		}
		if ( accno != null ) {
			IfaceToolbox.printXMLTagln( out, "ACCN", accno );
		}
		out.println( "  </study>" );
	}


	static void printInfo( PrintWriter out, MainStudies mainStudies, StudyFinder.StudyLocation info ) throws IOException {
		if ( info == null ) {
			return;
		}

		String study = info.getStudyUid();
		String mainStudy = mainStudies.getMainStudy( study, info.getPriorFilters(), info.getPriorValues() );
		
		out.println( "  <study>" );
		IfaceToolbox.printXMLTagln( out, "mainstudy", mainStudy );
		IfaceToolbox.printXMLTagln( out, "SYUI", study );
		IfaceToolbox.printXMLTagln( out, "ACCN", info.getAccessionNumber() );
		IfaceToolbox.printXMLTagln( out, "PANM", info.getPatientName() );
		
		// DEBUG ---
//		for ( Enumeration<?> e = info.getPriorValues().propertyNames(); e.hasMoreElements(); ) {
//			String fld = (String) e.nextElement();
//			String value = info.getPriorValues().getProperty( fld );
//			IfaceToolbox.printXMLTagln( out, fld, value );
//		}
		
		for ( String loc : info.getLocations() ) {
			IfaceToolbox.printXMLTagln( out, "location", loc );
		}
		out.println( "  </study>" );
	}


%><%

	IMSession is = new IMSession( request, response, out );
	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printXMLError( is, ErrorType.Permanent, "Has no registration info for this IP.", true );
		return;
	}

	StudyFinder sf = null;
	HashMap<String, StudyFinder.StudyLocation> locations = null;
	try {
		sf = new StudyFinder( is, true, true );
		locations = sf.collectStudyLocations( StudyFinder.StudyIdentifier.REQUEST, true );
	}
	catch ( IMIfaceException ex ) {
		IfaceToolbox.printXMLError( is, ErrorType.Permanent, ex.getMessage(), true );
		return;
	}
	
	IfaceToolbox.openXMLResult( is );
	out.println( " <content version=\"1\">" );

	MainStudies mainStudies = new MainStudies();
	
	Collection<String> accnos = sf.getAccessionNumberList();
	if ( accnos != null ) {
		for ( String accno : accnos ) {
			StudyFinder.StudyLocation info = locations.get( "ACCN:" + accno );
			if ( info == null ) {
				printEmpty( is.out, null, accno );
			}
			else {
				printInfo( is.out, mainStudies, info );
			}
		}
	}

	Collection<String> uids = sf.getStudyUIDList();
	if ( uids != null ) {
		for ( String uid : uids ) {
			StudyFinder.StudyLocation info = locations.get( "SYUI:" + uid );
			if ( info == null ) {
				printEmpty( is.out, uid, null );
			}
			else {
				printInfo( is.out, mainStudies, info );
			}
		}
	}
	
	out.println( " </content>" );
	IfaceToolbox.closeXMLResult( is );
	
	if ( true ) {
		// To avoid sending any other character on the connection after redirect
		return;
	}
%>