<%--
 * Component: iface
 *
 * Description:
 *   interface to initiate user related operations
 *
 * Parameters:
 *
 *		common Parameters:
 *				cOper           : operation ( "new", "update" or "delete" ) // undocumented: createOrUpdate
 *				cUser           : ID of the user to be created/edited/deleted
 *
 *		params for "new" only:
 *				cTemplateUser   : ID of a template user to copy settings from
 *
 *		params for "new" or "update":
 * 
 *				qe_URGT : Rights
 *				qe_UPWD : Password (Required parameter for 'new')
 *				qe_UNAM : User Name
 *				qe_MAIL : Email Address
 *				qe_UNOT : Notes about the user
 *				qe_ADDR : Address
 *				qe_UPH1 : Phone Number 1
 *				qe_UPH2 : Phone Number 2
 *				qe_UGRP : Group (Required parameter for 'new')
 *				qe_URAI : Radiologist ID
 *				qe_UTMZ : Time Zone
 *
 *		login expiration settings:
 *				cPWExpMode      : expiration mode:  'never' (default), 'days', 'logins' 
 *				cPWExpCount     : count of days or logins the password expires after
 *				cPWRevalidate   : how to revalidate: 'user', 'admin' (default)
 *				cPWChangeFirst  : change password after first login: 'true', 'false' - default is 'false' )
 * 
 *		plugin settings:
 *				cPlugin_<pluginname> : enable or disable viewer plugins for user ( 'on', 'off' )
 *
 *		location settings:
 *				cAddLoc           : the location row ID (given by operLocation.jsp?cOper=new) to add to this user's locations
 *				cLocDefault       : "true" means: also set as default for the user when adding
 *				cRemoveLoc        : the location row ID (given by operLocation.jsp?cOper=new) to remove from this user's locations
 *				cRemoveLocDelete  : "true" means if no user is referring to this Location, delete it, too
 *
 *		managed group settings:
 *				cAddManagedGroup  : the group id (name) to add to this user's managed groups
 *				cRemoveManagedGroup: the group id (name) to remove from this user's managed groups
 * 
 *
 *		not supported fields: 
 *				#qe_URGT : Rights - cUseGroupRights and cRights will be used instead
 *				#qe_UFLG : Flag - internal (????)
 *				#qe_UVER : Version - internal 
 *				#qe_UCOD : Code For New PW - internal (used in 'forgot password' emails)
 *				#qe_URES : Accessible Server - may be usable
 *		returns : none
 *
 *
--%><%@
	page import="java.io.*, java.security.NoSuchAlgorithmException, java.sql.*, java.util.*,
			imagemed.http.tabs.TabBean, imagemed.exception.*, imagemed.imsession.*, imagemed.imtables.*,
			imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType, imagemed.types.IMType,
			imagemed.types.flag.IMTypeur, imagemed.tables.*, imagemed.util.*"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"
	
%><%!

	static String[] DBFIELDS = new String[] { "URGT", "UPWD", "UGRP", "UNAM", "MAIL", "ADDR", "UPH1", "UPH2", "UNOT", "URAI", "UTMZ", "USAT" };

	private static void deleteUser( IMSession is ) throws IOException, SQLException {
		boolean isAdmin = is.hasRight( IMTypeur.ADMIN_RIGHT );
		
		String cID = is.request.getParameter( "cUser" );
		if ( cID == null ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "No user ID was specified" );
			return;
		}
		
		String group = IMAdminToolbox.getUserGroup( cID );

		if( !isAdmin && group != null && !is.isManaged( group ) ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "No right to delete user:" + cID + " in group:" + group );
			return;
		}

		if ( !IMAdminToolbox.deleteUser( is, cID ) ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "An error occured while deleting user: " + cID );
			return;
		}
		IfaceToolbox.printOK( is );
	}

	static enum MODE {
		CREATE,
		UPDATE,
		CREATEORUPDATE
	}
	
/* 	/**
	 * Checks parameters manipualting user rights and constructs Rights string to be passed to the backend to store
	 *
	 * @param is        session stuff with the HTTP request
	 * @param props     set calculated Rights string in this properties
	 * @param user      ID of the user to be added/modified
	 * @param group     Group ID for new users or if the user to be moved to a new group
	 * @param isNew     true indicates that a new user to be created
	 *
	 * @return true if and only if the Rights string could be successfully constructed and there were no right dependency problems
	 *  /
	private static boolean checkAndSetRights( IMSession is, Properties props, String user, String group, boolean isNew ) {
		 String useGroupRights = is.request.getParameter( "cUseGroupRights" );
		 if ( "true".equals( useGroupRights ) ) {
			 props.setProperty( "RIGHTS", IMTypeur.USE_GROUP_RIGHTS );
			 return true;
		 }
		 
		 String currentRights = "";
		 String currentGroup = null;
		 if ( !isNew ) {
			ResultSet rs = IMEnv.getDB().prepareStatement( "SELECT AGROUP, RIGHTS FROM " + IMEnvBase.getEnv( "ACCESSTBL" ) + " WHERE ID='" + userID + "'" ).executeQuery();
			if ( rs.next() ) {
				currentRights = rs.getString( "RIGHTS" );
				currentGroup = rs.getString( "AGROUP" );
			}
			rs.close();
		 }

		 String rights = is.request.getParameter( "cRights" );
		 if ( rights == null ) {
			 // No rights was specified - check if using group rights flag to be removed
			 if ( "false".equals( useGroupRights ) ) {
				 // Remove use group flag from current rights and set rights if changed
				 String newRights = imagemed.util.Helper.replace( currentRights, IMTypeur.USE_GROUP_RIGHTS, "" );
				 if ( ! newRights.equals( currentRights ) ) {
					 props.setProperty( "RIGHTS", newRights );
				 }
			 }
			 
			 // No rights to be changed
			 return true;
		 }

		 if ( currentRights.contains( IMTypeur.USE_GROUP_RIGHTS ) ) {
			 // Using group rights - checking if, using group rights is to be turned off 
			 if ( !"false".equals( useGroupRights ) ) {
				// changing rights will be ignored
				return true;
			 }
			 
			 // Remove use group rights flag from current rights 
			 currentRights = imagemed.util.Helper.replace( currentRights, IMTypeur.USE_GROUP_RIGHTS, "" );
		 }
		 
		 if ( rights.startsWith( "+" ) ) {
			 rights = addRights( currentRights, rights.substring( 1 ) );
		 }
		 else if ( rights.startsWith( "-" ) ) {
			 rights = removeRights( currentRights, rights.substring( 1 ) );
		 }

		Map<String, String> badRights = new HashMap<String, String>();
		Set<String> newRights = IMTypeur.checkRights( IMTypeur.rightsToSet( rigths ), badRights );
		
		if ( !badRights.isEmpty() ) {
			// There are incorrect rights - give a warning
		}
		
		newRights.remove( IMTypeur.SUPPORT_RIGHT );
		newRights.remove( "X" );
		 
		props.setProperty( "RIGHTS", IMTypeur.setTotRights( newRights ) );
		return true;
	}
 */
	
 	/**
 	 * Fills <code>userData</code> with actual data in the database for the user
 	 *
 	 * @param userID    user ID
 	 * @param userData  output map to add user data to
 	 *
 	 * @return true if the user exists and false if the user does not exist
 	 */
 	private static boolean fillUserData( String userID, Map<String, IMType.FldValue> userData ) {
		String accessTbl = IMEnvBase.getEnv( "ACCESSTBL" );
	 
		IMFieldManager fm = new IMFieldManager( IMEnvBase.getEnv( "FIELDSTBL" ), IMEnvBase.getEnv( "TYPESTBL" ), accessTbl );

		TableDescriptor tdesc = new TableDescriptor();
		TableParams tparams = new TableParams();
		tparams.setDefaultFields( DBFIELDS );

		Properties filters = new Properties();
		filters.setProperty( "USID", userID );
		tparams.setFilters( filters );
		tparams.setDefaultPageSize( -1 );

		tdesc.setTableParams( tparams );
		tdesc.setFieldManager( fm );
		SQLResult result = new SQLResult( tdesc );
		
		if ( !result.next() ) {
			result.close();
			return false;
		}

		for ( String fld : DBFIELDS ) {
			String value = result.getValue( fld );
			userData.put( fld, new IMType.FldValue( value ) );
		}
		
		result.close();
		return true;
	}
 
 
 	/**
 	 * Checks and saves password expiration settings
 	 *
 	 * @param is     Sesssion information including the HTTP request
 	 * @param user   User to update
 	 *
 	 * @return null on success or error message
	 */
 	private static String checkAndSavePWSettings ( IMSession is, String user ) {
 		String strExpire = is.request.getParameter( "cPWExpMode" );
 		String strCount = is.request.getParameter( "cPWExpCount" );
 		String strMode = is.request.getParameter( "cPWRevalidate" );
 		String strChange = is.request.getParameter( "cPWChangeFirst" );

 		if ( strExpire == null && strCount == null && strMode == null && strChange == null ) {
 			// nothing to do - return
 			return null;
 		}
 		
		return new PasswordManager( user ).modifyFromAPI( strExpire, strCount, strChange, strMode ) ;
 	}
 	 
 
	private static void changeUser( IMSession is, MODE mode ) throws IOException, SQLException, IMUserAdminException, NoSuchAlgorithmException {
		boolean isAdmin = is.hasRight( IMTypeur.ADMIN_RIGHT );
		
		String user = is.request.getParameter( "cUser" );
		if ( user == null ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cUser'." );
			return;
		}

		String templateUser = is.request.getParameter( "cTemplateUser" );
		String queryUser = user; // User to collect user data
		if ( mode == MODE.CREATE ) {
			if ( templateUser != null ) {
				queryUser = templateUser;
			}
		}
		else {
			// clear template user
			templateUser = null;
		}

		Map<String, IMType.FldValue> userData = new TreeMap<String, IMType.FldValue>(); 
		boolean isExisting = fillUserData( queryUser, userData );		

		if ( mode == MODE.CREATEORUPDATE ) {
			mode = isExisting ? MODE.UPDATE : MODE.CREATE;
		}

		if ( templateUser != null ) {
			if ( !isExisting ) {
				// Template user was not found - return error
				IfaceToolbox.printError( is, ErrorType.Permanent, "Error copying user" );
				return;
			}
			// creating a new user based on a template user
			// remove user specific fields from userData
			userData.remove( "UPWD" ); // Password 
			userData.remove( "UNAM" ); // User Name
			userData.remove( "MAIL" ); // Email Address
			userData.remove( "ADDR" ); // Address
			userData.remove( "UPH1" ); // Phone Number 1
			userData.remove( "UPH2" ); // Phone Number 2
		}
		
		String groupToBe = is.request.getParameter( "qe_UGRP" );
		if( !isAdmin ) {
			if( groupToBe != null && !is.isManaged( groupToBe ) ) {
				IfaceToolbox.printError( is, ErrorType.Permanent, "No right to put user into group:" + groupToBe );
				return;
			}
		
			if( mode == MODE.UPDATE ) {
				IMType.FldValue grVal = userData.get( "UGRP" );
				if( grVal != null ) {
					String group = grVal.getValue();
					if( group != null && !is.isManaged( group ) ) {
						IfaceToolbox.printError( is, ErrorType.Permanent, "No right to update user:" + user + " in group:" + group );
						return;
					}
				}
			}
		}
		
		Vector<String> warnings = new Vector<String>();
		
		for ( String fld : DBFIELDS ) {
			if ( is.request.getParameter( "qe_" + fld ) != null ) {
				IMType type = IMType.getTypeForColid( fld );
				IMType.FldResult res = type.setValue( userData, fld, is.request.getParameter( "qe_" + fld ) );
				if ( res.getCode() == IMType.RESULTCODE.ERROR ) {
					IfaceToolbox.printErrors( is, ErrorType.Permanent, res.getErrorMessages() );
					return;
				}
				else if ( res.getWarningMessages() != null && ! res.getWarningMessages().isEmpty() ) {
					warnings.addAll( res.getWarningMessages() );
				}
			}
			else if ( is.request.getParameter( "qm_" + fld ) != null ) {
				IMType type = IMType.getTypeForColid( fld );
				IMType.FldResult res = type.modifyValue( userData, fld, is.request.getParameter( "qm_" + fld ) );
				if ( res.getCode() == IMType.RESULTCODE.ERROR ) {
					IfaceToolbox.printErrors( is, ErrorType.Permanent, res.getErrorMessages() );
					return;
				}
				else if ( res.getWarningMessages() != null && ! res.getWarningMessages().isEmpty() ) {
					warnings.addAll( res.getWarningMessages() );
				}
			}
		}

		// Translate fields to properties accepted by IMAdminToolbox.changeUser()
		Properties props = new Properties();
		
		for ( String fld : userData.keySet() ) {
			IMType.FldValue fieldValue = userData.get( fld );
			if ( fieldValue == null ) {
				continue;
			}
			if ( !fieldValue.isChanged() && templateUser == null ) {
				continue;
			}
			if ( fieldValue.getValue() == null ) {
				continue;
			}
			
			props.setProperty( IMBaseFieldManager.staticGetDbName( fld ), fieldValue.getValue() );
		}
	
		String note = is.request.getParameter( "qe_UNOT" );
		if ( note != null ) {
			props.setProperty( "Note", note );
		}

		if ( ! props.isEmpty() || mode == MODE.CREATE ) {
			props.setProperty( "USERID", user ); 
			
			try {
				IMAdminToolbox.changeUser( is, props, mode == MODE.CREATE, templateUser );
			}
			catch ( IMUserAdminException iaEx ) {
				IfaceToolbox.printError( is, ErrorType.Permanent, "Error saving user data: " + iaEx.getMessage() );
				return;
			}
			catch ( Throwable th ) {
				IfaceToolbox.printError( is, ErrorType.Permanent, "An error occurred while saving user data" );
				return;
			}
		}
		
		String pwSettingResult = checkAndSavePWSettings( is, user );
		if ( pwSettingResult != null ) {
			warnings.add( pwSettingResult );
		}

		Properties plugins = new Properties();

		if ( templateUser != null ) {
			// creating a new user based on a template user
			// clear plugins by default
			String[] allPlugins = PluginToolbox.getAllPlugins();
			if ( allPlugins != null ) {
				for ( String pluginName : allPlugins ) {
					plugins.setProperty( pluginName, "false" );
				}
				try {
					PluginToolbox.setPluginsFor( user, plugins );
				}
				catch ( Exception ex ) {
//					warnings.add( "Plugin settings could not be saved: " + ex.getMessage() );
				}
				plugins.clear();
			}
		}

		for ( Enumeration<?> e = is.request.getParameterNames(); e.hasMoreElements(); ) {
			String parName = (String) e.nextElement();
			if ( !parName.startsWith( "cPlugin_" ) ) {
				continue;
			}
			String pluginName = parName.substring( 8 );
			String pluginValue = is.request.getParameter( parName );

			if ( "".equals( pluginName ) ) {
				warnings.add( "Missing plugin name in cPlugin parameter" );
				continue;
			}
			
			if ( "on".equals( pluginValue ) ) {
				plugins.setProperty( pluginName, "true" );
			}
			else if ( "off".equals( pluginValue ) ) {
				plugins.setProperty( pluginName, "false" );
			}
			else {
				warnings.add( "Unrecognized value was provided for plugin " + pluginName );
			}
		}

		if ( ! plugins.isEmpty() ) {
			try {
				PluginToolbox.setPluginsFor( user, plugins );
			}
			catch ( Exception ex ) {
				warnings.add( "Plugin settings could not be saved: " + ex.getMessage() );
			}
		}

		
		String addLoc = is.request.getParameter( "cAddLoc" );
		boolean isDefault = "true".equals( is.request.getParameter( "cLocDefault" ) );
		if( addLoc != null ) {
			try {
				LocationToolbox.addToUser( is, user, addLoc, isDefault );
			}
			catch( Throwable th ) {
				warnings.add( "Location couldn't be added to user: " + Helper.throwable2String( th ) );
			}
		}

		String remLoc = is.request.getParameter( "cRemoveLoc" );
		boolean deleteAlso = "true".equals( is.request.getParameter( "cRemoveLocDelete" ) );
		if( remLoc != null ) {
			try {
				LocationToolbox.removeFromUser( is, user, remLoc, deleteAlso );
			}
			catch( Throwable th ) {
				warnings.add( "Location couldn't be removed from user: " + Helper.throwable2String( th ) );
			}
		}

		String addManGroup = is.request.getParameter( "cAddManagedGroup" );
		if( addManGroup != null ) {
			try {
				IMAdminToolbox.addManagedGroupToUser( is, user, addManGroup );
			}
			catch( Throwable th ) {
				warnings.add( "Managed Group couldn't be added to user: " + Helper.throwable2String( th ) );
			}
		}

		String remManGroup = is.request.getParameter( "cRemoveManagedGroup" );
		if( remManGroup != null ) {
			try {
				IMAdminToolbox.removeManagedGroupFromUser( is, user, remManGroup );
			}
			catch( Throwable th ) {
				warnings.add( "Managed Group couldn't be removed from user: " + Helper.throwable2String( th ) );
			}
		}

		IfaceToolbox.printOK( is, warnings );
	}

%><%

	IMSession is = new IMSession( request, response, out );

	is.setNonCacheable(  );
	is.response.setContentType( "text/plain" );

	// in fact, this could be in RegOK(), but I didn't want to change too much things because of a concrete problem (CBT 2477)
	try {
		TabBean.checkLicense( is, false );
	}
	catch( IMException ie ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Restricted operation." );
		return;
	}

	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	} 

	if ( !is.hasRight( IMTypeur.USER_MANAGE_RIGHT ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to perform this operation." );
		return;
	}
	
	String operStr = request.getParameter( "cOper" );
	if ( operStr == null || operStr.length() == 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cOper'." );
		return;
	}

	is.sessionVariables.setProperty("httpIface", "true");

	try {
		if ( "delete".equals( operStr ) ) {
			deleteUser( is );
			return;
		}
		
		if ( "update".equals( operStr ) ) {
			changeUser( is, MODE.UPDATE );
			return;
		}
		
		if ( "new".equals( operStr ) ) {
			changeUser( is, MODE.CREATE );
			return;
		}

		if ( "createOrUpdate".equals( operStr ) ) {
			changeUser( is, MODE.CREATEORUPDATE );
			return;
		}
		
		IfaceToolbox.printError( is, ErrorType.Permanent, "Unspecified or unrecognized operation: " + operStr );
		return;
	}
	catch ( SQLException se ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "An SQLException occured during operation: " + se.getMessage() );
		return;
	}
	catch ( IOException ioe ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "An IOException occured during operation: " + ioe.getMessage() );
		return;
	}
	catch ( IMUserAdminException imuae ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "An error occured during operation: " + imuae.getMessage() );
		return;
	}
	catch ( RuntimeException re ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Error during operation: " + re.getMessage() );
		// return;
	}

	if ( true ) {
		return;
	}
%>