<%--
 * Component: iface
 *
 * Description:
 *   Retrieves the core data of the report and addenda
 *
 * Parameters:
 *
 *  @param cStudy           The Study IUID of the study
 *
 * For Type=Report or Addendum:
 *   Type, SR Instance UID, Interpretation ID, Status, HasDictation, No. of Key images, Note, Reserved, Author, Report Date, Transcriptionist,
 *   Transcription Date, Approver, Approval Date, Observation, Impression
 *
 * For Type=Image:
 *   Type, SOP IUID, Reference Number
 *
--%><%@
	page import="java.util.*,
			imagemed.imsession.*, imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType, imagemed.util.*"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%!

	static imagemed.types.enumeration.IMTypeISTI typeSI   = new imagemed.types.enumeration.IMTypeISTI();
	static imagemed.types.string.IMTypePN typePN = new imagemed.types.string.IMTypePN();
	static imagemed.types.enumeration.IMTypeni typeNormality = new imagemed.types.enumeration.IMTypeni();
	static imagemed.types.enumeration.IMTypeBRDS typeBiRADS = new imagemed.types.enumeration.IMTypeBRDS();

%><%

	IMSession is = new IMSession( request, response, out );
	
	// Check login
	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	}

	String parStudy = request.getParameter( "cStudy" );
	if ( parStudy == null ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cStudy'." );
		return;
	}
	

	String redirHost = RedirectToolbox.getRedirectForStudy( is, parStudy );
	if ( redirHost != null ) {
		String redirURL = redirHost + "/iface/report.jsp?" + request.getQueryString();
		response.sendRedirect( redirURL );
		return;
	}

	boolean bRight = false;

	if ( is.regRights != null )
		bRight = is.regRights.indexOf( "R" ) != -1;

	String studyDirName = RepositoryHandler.getLocation( RepositoryHandler.DATA, parStudy );
	if ( studyDirName == null ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Study was not found.");
		return;
	}

	ReportToolbox.moveOldReportToDicomData( studyDirName, parStudy );

	if ( !bRight && is.regRights != null ) {
		// Lock is enabled with transcription and/or edit rights
		// remove these rights
		is.regRights = Helper.replace( Helper.replace( is.regRights, "E", "" ), "T", "" );
	}

	Properties prpHeaders = new Properties();
	Properties prpTimes = Report.getReportModified( is, parStudy, "None", prpHeaders );


	int nWaitTime = 30000;
	int nElapsed = 0;

	while ( ReportToolbox.hasNewerReport( prpTimes, parStudy ) && nElapsed < nWaitTime ) {
		Thread.sleep( 50 );
		nElapsed += 50;
	}

	if ( ReportToolbox.hasNewerReport( prpTimes, parStudy ) ) {
		IfaceToolbox.printError( is, ErrorType.Transient, "The report is not up to date, was modified." );
		return;
	}

	Properties results     = new Properties();
	Vector<Properties> interpretations = new Vector<Properties>();
	Vector<Properties> externals       = new Vector<Properties>();

	if ( is.sessionVariables == null )
		is.sessionVariables = new Properties();
	is.sessionVariables.setProperty( "study", parStudy );
	
	try {
		StoreState storestate = new StoreState( parStudy );
		storestate.load();

		String sourceStudy = storestate.getSourceStudy( true );
		is.sessionVariables.setProperty( "sourceStudy", sourceStudy );
	}
	catch ( Throwable th ) {
	}


	int res = ReportToolbox.parseReport( is, results, interpretations, externals );
	if ( res == ReportToolbox.PR_INVALID_VERSION ) {
		// This should not be happen with new (SR) reports
		// itt meg lehetne hivni a convertert, es ujra probalkozni
		IfaceToolbox.printError( is, ErrorType.Permanent, "Invalid report version." );
		return;
	}
	else if ( res == ReportToolbox.PR_ERROR ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Could not read report." );
		return;
	}

	String impression = results.getProperty( "impression", "" );
	Properties prpImages = new Properties();
	
	
	response.setContentType( "text/plain" );
	IfaceToolbox.printOK( is );

	for ( int i = 0; i < interpretations.size(); i++ ) {
		Properties interpretation = interpretations.elementAt( i );

		String sriuid   = interpretation.getProperty( "sriuid", "" );   // SR SOP Instance UID
		String id       = interpretation.getProperty( "id", "" );
		String status   = interpretation.getProperty( "statid", "CREATED" );
		String recdate  = interpretation.getProperty( "recdate", "" );
		String rectime  = interpretation.getProperty( "rectime", "" );
		String tscdate  = interpretation.getProperty( "tscdate", "" );
		String tsctime  = interpretation.getProperty( "tsctime", "" );
		String tscriber = interpretation.getProperty( "tscriber", "" );
		String author   = interpretation.getProperty( "author", "" );
		String appdate  = interpretation.getProperty( "appdate", "" );
		String apptime  = interpretation.getProperty( "apptime", "" );
		String approver = interpretation.getProperty( "approver", "" );
		String text     = interpretation.getProperty( "text", "" );
		String dictuid  = interpretation.getProperty( "dictuid" );
		String figure   = interpretation.getProperty( "figure", "" );
		String note     = interpretation.getProperty( "note", "" );
		String authorid = interpretation.getProperty( "authorid", "" );
		String tscriberid = interpretation.getProperty( "tscriberid", "" );
		String approverid = interpretation.getProperty( "approverid", "" );
		String location = interpretation.getProperty( "location", "" );
		String locationName = interpretation.getProperty( "locationName", "" );
		String locationAddress = interpretation.getProperty( "locationAddress", "" );
		String normality = interpretation.getProperty( "normality", "" );
		String birads = interpretation.getProperty( "birads", "" );

		// Type
		out.print( ( "type".equals( "REPORT" ) || i == 0 ? "Report" : "Addendum" ) );
		// SR Instance UID
		out.print( "\t" + sriuid );
		// ID
		out.print( "\t" + id );
		// Status
		out.print( "\t" + typeSI.displayText( status ) );
		// HasDictation
		out.print( "\t" + ( dictuid == null ? "0" : "1" ) );

		// No. of Key images
		int keyImages = 0;
		for ( ValTokenizer images = new ValTokenizer( figure, '\n' ); images.hasMoreTokens(); ) {
			String line = images.nextToken();
			if ( line.startsWith( "ID" ) )
				keyImages++;
		}
		out.print( "\t" + keyImages );

		// Note (formerly known as Res1)
		out.print( "\t" + Helper.encodeProcessable( note ) );
		// Normality (formerly known as Reserved and Res2)
		out.print( "\t" + typeNormality.displayText( normality ) );
		// Author
		out.print( "\t" + typePN.displayText( author ) );
		// Report Date
		out.print( "\t" + recdate + " " + rectime );
		// Transcriptionist
		out.print( "\t" + typePN.displayText( tscriber ) );
		// Transcription Date
		out.print( "\t" + tscdate + " " + tsctime );
		// Approver
		out.print( "\t" + typePN.displayText( approver ) );
		// Approval Date
		out.print( "\t" + appdate + " " + apptime );
		// Observation
		out.print( "\t" + Helper.encodeProcessable( text ) );
		// Impression
		if ( i == 0 ) {
			out.print( "\t" + Helper.encodeProcessable( impression ) );
		}
		else {
			out.print( "\t" + Helper.encodeProcessable( null ) );
		}
		// Author ID
		out.print( "\t" + Helper.encodeProcessable( authorid ) );
		// Transcriptionist ID
		out.print( "\t" + Helper.encodeProcessable( tscriberid ) );
		// Approver ID
		out.print( "\t" + Helper.encodeProcessable( approverid ) );
		// BiRADS code
		out.print( "\t" + Helper.encodeProcessable( birads ) );
		// Reading location registered ID
		out.print( "\t" + Helper.encodeProcessable( location ) );
		// Reading location name
		out.print( "\t" + Helper.encodeProcessable( locationName ) );
		// Reading location address
		out.print( "\t" + Helper.encodeProcessable( locationAddress ) );
		out.println();

		// Process Figure
		for ( ValTokenizer images = new ValTokenizer( figure, '\n' ); images.hasMoreTokens(); ) {
			String line = images.nextToken();
			if ( line.startsWith( "ID" ) ) {
				String sopi = line.substring( 3 ).trim();
				String frameNum = "1";
				int p = sopi.indexOf( "/" );
				int q = sopi.indexOf( ".", p );
				int r = sopi.indexOf( "_", p );
				if ( p != -1 && q != -1 && r != -1 ) {
					frameNum = sopi.substring( r + 1 );
					sopi = sopi.substring( q + 1, r );
				}

				// Type
				out.print( "Image" );
				// SOP IUID
				out.print( "\t" + sopi );
				// Frame Number
				out.print( "\t" + frameNum );
				// Reference Number
				int refNum = 0;
				try {
					refNum = Integer.parseInt( prpImages.getProperty( sopi, "0" ) );
				}
				catch ( Throwable th ) {
				}
				refNum++;
				out.print( "\t" + refNum + "\n" );
				prpImages.setProperty( sopi, "" + refNum );
			}
		}
	}
%>