<%--
 * Component: iface
 *
 * Description:
 *   interface to initiate user related operations
 *
 * Parameters:
 *
 *		common Parameters:
 *				cOper         	: operation ("change" or "delete")
 *		params for "change":
 *				c_ID          	: identifies the user to be created/edited
 *				c_PASSWD      	: password of the user
 *				c_Notes       	: notes about the user
 *				c_RIGHTS      	: rights for the user (like 'ASQ')
 *				c_ADDRESS     	: address of the user
 *				c_AGROUP      	: group of user, one of {Admin, Generalist, Physician, Radiologist, Technologist}
 *				c_EMAIL       	: e-mail address of user
 *				c_PHONE2      	: phone number #2 of the user
 *				c_PHONE1      	: phone number #1 of the user
 *				c_USERNAME    	: name of the user
 *				c_USRAUTHTARGET : authentication agent to be used for the user ('EP', 'SYSTEM', 'GROUP' or one of the LDAP configuration IDs listed by ldapConfigs.jsp)
 *		returns : none
 *
 *		params for "delete":
 *				cID           : identifies the user to be deleted
 *		returns : none
 *
--%><%@
	page import="java.io.*, java.sql.*, java.security.NoSuchAlgorithmException, java.util.*,
			imagemed.imsession.*, imagemed.exception.*, imagemed.jsputil.*, imagemed.jsputil.IfaceToolbox.ErrorType"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"
	
%><%!

	/**
	 *	this method returns true if the user has admin rights or is identical to the given one
	 *	(for example he want's to alter his own folder or has admin right to alter someone else's...)
	 */
	private boolean hasRight( IMSession is, String user ) throws IOException {
		if ( IMAdminToolbox.bAdminRight( is ) ) {
			return true;
		}
		if ( user != null && user.equals( is.regID ) ) {
			return true;
		}
		
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no right to perform this operation." );
		return false;
	}

	private void deleteUser( IMSession is ) throws IOException, SQLException {
		String cID = is.request.getParameter( "c_ID" );
		if ( !hasRight( is, null ) ) {
			return;
		}
		if ( !IMAdminToolbox.deleteUser( is, cID ) ) {
			IfaceToolbox.printError( is, ErrorType.Permanent, "An error occured while deleting user: " + cID );
			return;
		}
		IfaceToolbox.printOK( is );
	}

	private void changeUser( IMSession is ) throws IOException, Exception, IMUserAdminException, NoSuchAlgorithmException {
		String cID = is.request.getParameter( "c_ID" );
		if ( !hasRight( is, null ) ) {
			return;
		}
		boolean isNew = !IMAdminToolbox.existUser( cID, IMEnvBase.getEnv( "ACCESSTBL" ) );
		Properties props = IMAdminToolbox.makeProp( is.request, "c_" );
		props.setProperty( "USERID", cID );
		props.remove( "ID" );
		IMAdminToolbox.changeUser( is, props, isNew, null );
		IfaceToolbox.printOK( is );
	}

%><%

	IMSession is = new IMSession( request, response, out );

	is.response.setContentType( "text/plain" );

	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Has no registration info for this IP." );
		return;
	}

	String operStr = request.getParameter( "cOper" );
	if ( operStr == null || operStr.length() == 0 ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Missing parameter: 'cOper'." );
		return;
	}

	try {
		if ( "delete".equals( operStr ) ) {
			deleteUser( is );
			return;
		}
		if ( "change".equals( operStr ) ) {
			changeUser( is );
			return;
		}

		IfaceToolbox.printError( is, ErrorType.Permanent, "Unrecognized operation: '" + operStr + "'." );
		return;
	}
	catch ( SQLException se ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "An SQLException occured during operation: " + se.getMessage() );
		return;
	}
	catch ( IOException ioe ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "An IOException occured during operation: " + ioe.getMessage() );
		return;
	}
	catch ( IMUserAdminException imuae ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "An error occured during operation: " + imuae.getMessage() );
		return;
	}
	catch ( RuntimeException re ) {
		IfaceToolbox.printError( is, ErrorType.Permanent, "Error during operation: " + re.getMessage() );
		return;
	}
%>